@@ -8,11 +8,16 @@ from .group_tree.core import make
 from .group_tree.atom import Atom, Mem, Memstr, Memnone
 from .group_tree.rule import Rule, SELF
 from .group_tree.file import File, VFile, IFile
-from .group_tree.graphviz import print_graphviz
-from .group_tree.misc import print_method
+from .group_tree.tools.graphviz import print_graphviz
+from .group_tree.tools.mermaid import print_mermaid
+from .group_tree.tools.misc import (
+    print_method,
+    print_dirtree,
+    stringify_dirtree,
+)
 from .core.make import MakeSummary
 
-VERSION = "0.4.2"
+VERSION = "0.5.0"
 
 __all__ = [
     "SELF",
@@ -29,7 +34,10 @@ __all__ = [
     "VFile",
     "IFile",
     "print_graphviz",
+    "print_mermaid",
     "print_method",
     "MakeSummary",
     "Rule",
+    "print_dirtree",
+    "stringify_dirtree",
 ]
