@@ -1,6 +1,7 @@
+from multiprocessing.pool import Pool
 import traceback
 import enum
-from typing import Callable, List, NamedTuple, Set, Sequence
+from typing import Callable, List, NamedTuple, Optional, Set, Sequence, TypeVar
 
 from . import events
 from .abc import IRule, IEvent, UpdateResults
@@ -44,12 +45,15 @@ def _toplogical_sort(
     return res
 
 
+_T_Rule = TypeVar("_T_Rule", bound=IRule)
+
+
 def make(
-    id2rule: Sequence[IRule],
+    id2rule: Sequence[_T_Rule],
     ids: Sequence[int],
     dry_run: bool,
     keep_going: bool,
-    callback: Callable[[IEvent], None],
+    callback: Callable[[IEvent[_T_Rule]], None],
 ):
     if len(ids) == 0:
         return MakeSummary(total=0, update=0, skip=0, fail=0, discard=0)
@@ -74,7 +78,7 @@ def make(
 
         try:
             result = process_rule(
-                r, dry_run, par_updated, i in main_ids, callback
+                r, dry_run, par_updated, i in main_ids, callback, None
             )
         except Exception as e:
             result = Result.Fatal
@@ -113,11 +117,12 @@ def make(
 
 
 def process_rule(
-    rule: IRule,
+    rule: _T_Rule,
     dry_run: bool,
     par_updated: bool,
     is_main: bool,
-    callback: Callable[[IEvent], None],
+    callback: Callable[[IEvent[_T_Rule]], None],
+    pool: Optional[Pool],
 ):
     if dry_run:
         res = rule.check_update(par_updated, True)
@@ -148,27 +153,31 @@ def process_rule(
     callback(events.Start(rule))
 
     try:
-        rule.preprocess(callback)
+        rule.preprocess()
     except Exception as e:
         callback(events.PreProcError(rule, e))
         return Result.Fail
 
     try:
-        rule.method(*rule.args, **rule.kwargs)
-        succ = True
+        if pool is None:
+            rule.method()
+            succ = True
+        else:
+            pool.apply(rule.method)
+            succ = True
     except Exception as e:
         callback(events.ExecError(rule, e))
         succ = False
     except KeyboardInterrupt:
         try:
-            rule.postprocess(callback, False)
+            rule.postprocess(False)
         except Exception:
             pass
 
         raise KeyboardInterrupt()
 
     try:
-        rule.postprocess(callback, succ)
+        rule.postprocess(succ)
     except Exception as e:
         callback(events.PostProcError(rule, e))
         return Result.Fail
