@@ -52,6 +52,9 @@ class INode(metaclass=ABCMeta):
         else:
             return "/" + "/".join(self.name_tuple)
 
+    def __repr__(self) -> str:
+        return f"{type(self)}(name={self.name})"
+
 
 T_Self = TypeVar("T_Self", bound="IGroup")
 
@@ -84,11 +87,70 @@ class IGroup(INode, metaclass=ABCMeta):
     def set_prefix(
         self: T_Self, dirname: object = None, *, prefix: object = None
     ) -> T_Self:
+        """
+        Set the path prefix of this group.
+
+        Args:
+            dirname: if specified, prefix will be ``dirname + "/"``
+            prefix: path prefix.
+
+        You must specify either but not both of ``dirname`` or ``prefix``.
+
+        ``self.set_prefix("a")`` is equivalent to
+        ``self.set_prefix(prefix="a/")``.
+
+        If this group is not the root group and the given prefix is a
+        relative path, the path prefix of the parent group will be added to
+        its start. Absolute paths do not undergo this prefixing.
+
+        Note:
+            This method may be called only when the prefix is not yet
+            determined. i.e. You may NOT call this method whenever,
+
+            * You have created this group as a root group
+            * You have once called it
+            * You have once read :attr:`self.prefix <.prefix>`:
+              reading :attr:`self.prefix <.prefix>` internally finalizes
+              the prefix to ``"{name of this group}/"``
+            * You have once read the prefix of a child group:
+              reading a child's prefix internally reads the parent's prefix
+            * You have initialized any rule in the sub-tree:
+              initializing a rule internally reads its parent's prefix
+
+        Example:
+
+            .. testcode::
+
+                # (For Unix only)
+
+                from jtcmake import UntypedGroup
+
+                g = UntypedGroup("root")
+
+                g.add_group("foo").set_prefix("foo-dir")  # dirname
+                g.add_group("bar").set_prefix(prefix="bar-")  # prefix
+                g.add_group("baz").set_prefix("/tmp/baz")  # dirname abspath
+                g.add_group("qux")  # no explicit setting
+
+                assert g.prefix == "root/"
+                assert g.foo.prefix == "root/foo-dir/"
+                assert g.bar.prefix == "root/bar-"
+                assert g.baz.prefix == "/tmp/baz/"
+                assert g.qux.prefix == "root/qux/"
+        """
         if self.__prefix is not None:
             raise Exception(
-                f'Prefix is already set (to "{self.__prefix}") and '
-                "may not be overwritten. Make sure to call set_prefix "
-                "before initializing child groups and rules. "
+                f'Prefix is already set (to "{self.__prefix}"). '
+                "This method may be called only when the prefix is not yet "
+                "determined. i.e. You may NOT call this method whenever,\n"
+                "* You have created this group as a root group\n"
+                "* You have once called it\n"
+                "* You have once read `self.prefix`\n"
+                "  * reading `self.prefix` internally finalizes the prefix\n"
+                "* You have once read the prefix of a child group\n"
+                "  * it internally reference the parent's prefix\n"
+                "* You have initialized any rule in the sub-tree\n"
+                "  * initializing a rule internally reads its parent's prefix"
             )
 
         p = parse_args_prefix(dirname, prefix)
@@ -100,9 +162,15 @@ class IGroup(INode, metaclass=ABCMeta):
 
         return self
 
-    @final
     @property
+    @final
     def prefix(self) -> str:
+        """
+        Path prefix of this group.
+
+        Seealso:
+            :func:`set_prefix`
+        """
         if self.__prefix is None:
             # Root node must get prefix in __init__
             assert self.parent != self
@@ -116,11 +184,23 @@ class IGroup(INode, metaclass=ABCMeta):
     @property
     @abstractmethod
     def groups(self) -> Mapping[str, IGroup]:
+        """
+        Readonly dictionary of child groups.
+
+        * Key: base name of the child group
+        * Value: child group node object
+        """
         ...
 
     @property
     @abstractmethod
     def rules(self) -> Mapping[str, IRule]:
+        """
+        Readonly dictionary of child rules.
+
+        * Key: base name of the child rule
+        * Value: child rule node object
+        """
         ...
 
 
@@ -133,11 +213,20 @@ class IRule(INode, metaclass=ABCMeta):
     @property
     @abstractmethod
     def files(self) -> Mapping[str, IFile]:
+        """
+        Readonly dictionary of output files.
+
+        * Key: file key of the file
+        * Value: :class:`jtcmake.IFile` object
+        """
         ...
 
     @property
     @abstractmethod
     def xfiles(self) -> Collection[str]:
+        """
+        List of path of the input files.
+        """
         ...
 
     @abstractmethod
@@ -268,22 +357,28 @@ class GroupTreeInfo:
         "memo_factory",
         "memo_store",
         "rules_to_be_init",
+        "root",
     )
 
     rule_store: RuleStore
     logwriter: IWriter
-    memo_factory: Callable[[object], IMemo]
+    memo_factory: Callable[[Path, object], IMemo]
     memo_store: Dict[int, IAtom]
     rules_to_be_init: Set[Tuple[str, ...]]
+    root: IGroup
 
     def __init__(
-        self, logwriter: IWriter, memo_factory: Callable[[object], IMemo]
+        self,
+        logwriter: IWriter,
+        memo_factory: Callable[[Path, object], IMemo],
+        root: IGroup,
     ):
         self.logwriter = logwriter
         self.memo_factory = memo_factory
         self.memo_store = {}
         self.rule_store = RuleStore()
         self.rules_to_be_init = set()
+        self.root = root
 
 
 P = ParamSpec("P")
@@ -308,6 +403,9 @@ def require_tree_init(
             )
         return method(self, *args, **kwargs)
 
+    _method.__annotations__ = method.__annotations__
+    _method.__doc__ = method.__doc__
+
     return _method
 
 
@@ -332,29 +430,25 @@ def make(
             Maximum number of rules that can be made concurrently.
             Defaults to 1 (single process, single thread).
 
-            Note that safely using njobs >= 2 and fully exploiting the power
-            of multi-core processors require a certain level of
-            understanding of Python's threading and multiprocessing.
+    Warning:
 
-            Each rule is made on a child process if it is transferable.
-            A rule is "transferable" if both of the following conditions
-            are met:
+        Safely and effectively using njobs >= 2 require a certain level of
+        understanding of Python's threading and multiprocessing and their
+        complications.
 
-            - method/args/kwargs of the rule are all picklable
-            - Pickle representation of method/args/kwargs created in the
-              main process is unpicklable in child processes
+        Only *inter-process transferable* rules are executed on child processes.
+        Other rules are executed on threads of the main process, thus
+        subject to the constraints of global interpreter lock (GIL).
 
-            If a rule is not transferable, it is made on a sub-thread of
-            the main process. Thus the method must be thread-safe. Also note
-            that methods running on the main process are subject to the
-            global interpreter lock (GIL) constraints.
+        *inter-process transferable* means being able to be sent to a child
+        process without errors.
 
-            Child processes are started by the 'spawn' method, not 'fork',
-            even on Linux systems.
-            njobs >= 2 may not work on interactive interpreters.
-            It should work on Jupyter Notebook/Lab but any function or class
-            that are defined on the notebook is not transferable and thus
-            executed in the main process.
+        Child processes are started by the 'spawn' method, not 'fork',
+        even on Linux systems.
+
+        njobs >= 2 may not work on interactive interpreters.
+        It should work on Jupyter Notebook/Lab but any function or class
+        that are defined on the notebook is not inter-process transferable.
     """
     if len(rule_or_groups) == 0:
         return MakeSummary(total=0, update=0, skip=0, fail=0, discard=0)
